/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <htc.h>            /* HiTech General Includes */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "i2c.h"
#include "OLIMEX.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */

void InitApp(void)
{
    /* TODO Initialize User Ports/Peripherals/Project here */

    /* Setup analog functionality and port direction */

	ANSELA = 0;
	ANSELC = 0;

	JMP_TRIS = 1;
	SDA_LAT = 0;
	SCL_LAT = 0;
	SDA_TRIS = 1;
	SCL_TRIS = 1;

	DE_TRIS = 0;
	nRE_TRIS = 0;
	R_TRIS = 1;
	D_TRIS = 0;

	TXD_TRIS = 1;
	RXD_TRIS = 1;






    /* Initialize peripherals */

	/*CLC Setup*/
	APFCONbits.CLC1SEL = 1;
	NCO1CLKbits.N1CKS = 11;
	NCO1CONbits.N1EN = 1;


	// File: CLC2.inc
	// Generated by CLC Designer, Version: 1.0.0.3
	// Date: 21/04/2012 11:05
	// Device:PIC16(L)F1503

	CLC1GLS0 = 0x02;
	CLC1GLS1 = 0x00;
	CLC1GLS2 = 0x00;
	CLC1GLS3 = 0x00;
	CLC1SEL0 = 0x01;
	CLC1SEL1 = 0x00;
	CLC1POL  = 0x02;
	CLC1CON  = 0x00;

	CLC2GLS0 = 0x02;
	CLC2GLS1 = 0x00;
	CLC2GLS2 = 0x00;
	CLC2GLS3 = 0x00;
	CLC2SEL0 = 0x01;
	CLC2SEL1 = 0x00;
	CLC2POL  = 0x02;
	CLC2CON  = 0x00;

    /* Enable interrupts */
}
void InitAddress(void)
{
	char address;
	/* READ THE JUMPER*/
	if(!JMP_PORT)
	{
		/* SET ADDRESS FOR PROGRAMMING*/
		ADDRESS = 0xF0;
		return;
	}
	else
	{
		address = ReadFlash(ADDR_MEM);
		/* IF THE MEMORY CELL IS EMPTY*/
		if (address == 0xFF)
			ADDRESS = ADDR_DFL;
		else
			ADDRESS = address;
	}
}
char ReadFlash(short address)
{
	char data = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.RD = 1;
	Nop();
	Nop();

	data = PMDATL;
	return data;
}
void EraseFlash(short address)
{
	GIE = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.FREE = 1;
	PMCON1bits.WREN = 1;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}
void UnlockFlash(void)
{
	PMCON2 = 0x55;
	PMCON2 = 0xAA;
	PMCON1bits.WR = 1;
	Nop();
	Nop();
}
void WriteFlash(short data, short address)
{
	GIE = 0;
	PMCON1bits.CFGS = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.FREE = 0;
	PMCON1bits.LWLO = 1;
	PMCON1bits.WREN = 1;
	PMDATL = data & 0x00FF;
	data >>= 8;
	PMDATH = data & 0x00FF;
	PMCON1bits.LWLO = 0;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}
void CommandSetCLC()
{
	char data, read, write;
	data = ReadByteI2C();
	SendAck();

	
	write = (data & 0x01) ? 1 : 0 ;
	read = (data & 0x02) ? 1 : 0 ;
	

	if(write)
	{
		/*Write Enable*/
		DE_LAT = 1;
		CLC1CONbits.LC1EN = 1;
		CLC1CONbits.LC1OE = 1;
	}
	else
	{
		/*Write Disable*/
		DE_LAT = 0;
		CLC1CONbits.LC1EN = 0;
		CLC1CONbits.LC1OE = 0;

	}

	if(read)
	{
		/*Read Enable*/
		RXD_TRIS = 0;
		nRE_LAT = 0;
		CLC2CONbits.LC2EN = 1;
		CLC2CONbits.LC2OE = 1;
	}
	else
	{
		/*Read Disable*/
		RXD_TRIS = 1;
		nRE_LAT = 1;
		CLC2CONbits.LC2EN = 0;
		CLC2CONbits.LC2OE = 0;
	}



}
void CommandSetAddress()
{
	char address;
	address = ReadByteI2C();
	SendAck();

	/* STORE THE NEW ADDRESS INTO THE FLASH MEMORY */
	EraseFlash(ADDR_MEM);
	WriteFlash(address, ADDR_MEM);
	ADDRESS = address;
}


void StartSystem(void)
{
    char company, device, address, command;

    /*Loop until START condition is recieved*/
    StartI2C();

    /*Read the first bit of the address*/
    company = ReadByteI2C();
    WriteFlash(0x3FFF, 0x7FF);
    if (company == OLIMEX)
    {
	    SendAck();

	    /*Read the second bit of the address*/
	    device = ReadByteI2C();
	    WriteFlash(0x3FFF, 0x7FF);
	    if (device == MOD_RS485_ISO)
	    {
		    SendAck();

		    /*Read the third bit of the address*/
		    address = ReadByteI2C();
		    WriteFlash(0x3FFF, 0x7FF);
		    if(address == ADDRESS)
		    {
			    SendAck();
			    command = ReadByteI2C();
			    WriteFlash(0x3FFF, 0x7FF);
			    SendAck();
			    switch(command)
			    {
			    case SET_CLC:	CommandSetCLC();	break;
			    case SET_ADDRESS:	CommandSetAddress();	break;

			    default: break;
			    }
		    }
		    else
			    SendNack();
	    }
	    else
		    SendNack();
    }
    else
	    SendNack();
    /*Wait for STOP condition*/
    StopI2C();




    }

