/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <p18f26j50.h>            /* HiTech General Includes */
#include <flash.h>
#include <stdlib.h>

#include "user.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/
void StartI2C(void)
{
	Delay();
	SDA_TRIS = 0;
	Delay();
	Delay();
	Delay();
	SCL_TRIS = 0;
	Delay();
	Delay();
	Delay();
}
void StopI2C(void)
{
	Delay();
	Delay();
	Delay();
	SDA_TRIS = 0;
	Delay();
	Delay();
	Delay();
	SCL_TRIS = 1;
	Delay();
	Delay();
	Delay();
	SDA_TRIS = 1;
	Delay();	
}
char WriteByte(short data)
{
	int i = 8,ack;
	while(i--){
		Delay();
		Delay();
		SDA_TRIS = (data & 0x80) ? 1 : 0;
		data <<= 1;
		Delay();
		Delay();
		SCL_TRIS = 1;
		Delay();
		Delay();
		Delay();
		Delay();
		SCL_TRIS = 0;
		if(i==0) break;
	}

//-------READ ACK---------
	Delay();
	Delay();
	SDA_TRIS = 1;
	Delay();
	Delay();
	SCL_TRIS = 1;
	Delay();
	Delay();
	ack = SDA_PORT;
	Delay();
	Delay();
	SCL_TRIS = 0;
	Delay();
	Delay();

	return ack;

}
char ReadByte(void)
{
	char data = 0;
	char i;
	SDA_TRIS = 1;
	for(i = 0; i <8; i++)
	{
		Delay();
		Delay();
		SCL_TRIS = 1;
		Delay();
		Delay();
		data |= SDA_PORT & 0x01;
		if(i != 7) data <<= 1;
		Delay();
		Delay();
		SCL_TRIS = 0;
		Delay();
		Delay();


	}
	return data;
}
int ReadWord(void)
{
	int data = 0;
	int i;
	SDA_TRIS = 1;
	for(i = 0; i <16; i++)
	{
		Delay();
		Delay();
		SCL_TRIS = 1;
		while(SCL_PORT == 0);	//CLOCK STRETCHING
		Delay();
		Delay();
		data |= SDA_PORT & 0x01;
		if(i != 15) data <<= 1;
		Delay();
		Delay();
		SCL_TRIS = 0;
		Delay();
		Delay();


	}
	return data;
}
void Delay(void)
{
	unsigned int i = 300;
	while(i--){
		if(i == 1) break;
	}
}
void DelayMs(void)
{
	unsigned int i = 30000;
	while(i--){
		if(i == 1) break;
	}
}
void ScanAddresses()
{
//	char i = 0xF0, ack, j = 0;
//	Erase_Flash();
//	SendCommand(0x00,0x00);
//	while(j < 0x5){
//		ack = SendCommand(i,0x00);		
//		if(!ack)WriteWordFlash(0x6010 + 2*(j),i);
//		j++;
//		i++;
//	}
}
char ScanDevices()
{
	int i;
	char ack, count = 0;
	


	


	for(i = 0; i < 10; i++)
	{
		DevicesFound[count].ID = 0;
		StartI2C();
		Delay();
		WriteByte(0x90);
		Delay();
		WriteByte(0x04);
		Delay();
		ack = WriteByte(0xB0+i);
		Delay();
		WriteByte(0x00);
		Delay();
		StopI2C();
		Delay();
		Delay();

		if(!ack)
		{
			DevicesFound[count].Company = 0x90;
			DevicesFound[count].Device = 0x04;
			DevicesFound[count].Address = 0xB0+i;
			DevicesFound[count].ID = count+1;
			count++;

		}

	}
	return count;
}

char ScanAddress(char address)
{
	char ack;

	StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x04);
	Delay();
	ack = WriteByte(address);
	Delay();
	WriteByte(0x00);
	Delay();
	StopI2C();
	Delay();
	Delay();

	return ack;

}
char CommandSetAddress(char data)
{
	char ack;
	StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x04);
	Delay();
	WriteByte(0xF0);
	Delay();
	WriteByte(0xB0);
	Delay();
	ack = WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();

	return ack;

}
char FirstFree()
{
//	unsigned short *data;
	char addr = 0xF0;
//	while(addr < 0xF5){
//		ReadFlash(0x6010 + 2*(addr-0xF0),2,data);
//		if(*data == 0xFFFF) break;
//		addr++;
//	}
//	WriteWordFlash(0x6050,addr);
	return addr;
}
void Erase_Flash()
{
	DWORD_VAL  addr;

	addr.Val = 0x6000 ;
	TBLPTRU = addr.byte.UB;
	TBLPTRH = addr.byte.HB;
	TBLPTRL = addr.byte.LB;

	EECON1bits.WREN = 1;
	EECON1bits.FREE = 1;
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0x0AA;
	EECON1bits.WR = 1;
	INTCONbits.GIE = 1;
}
void InitApp(void)
{
    
	//BUTTON
	TRISBbits.TRISB0 = 1;
	INTCON2bits.RBPU = 0;
	ANCON1bits.PCFG12 = 1;

	//I/O Pins
	TRISA &= (~0x2F);
	LATA |= 0x2f;
	TRISC &= (~0x07);
	LATC |= 0x07;
	
	TRISCbits.TRISC6 = 0;
	TRISCbits.TRISC7 = 1;

	SDA_TRIS = 1;
	SCL_TRIS = 1;
	SDA_LAT = 0;
	SCL_LAT = 0;

}
void CommandSetTris(char data)
{
	 StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x01);
	Delay();
	WriteByte(0xA0);
	Delay();
	WriteByte(0x01);
	Delay();
	WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();
}
void CommandSetLat(char data)
{
	 StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x01);
	Delay();
	WriteByte(0xA0);
	Delay();
	WriteByte(0x02);
	Delay();
	WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();
}
char CommandGetPort()
{
	char port;
	StartI2C();
	Delay();
	 WriteByte(0x90);
	Delay();
	WriteByte(0x01);
	Delay();
	WriteByte(0xA0);
	Delay();
	WriteByte(0x03);
	Delay();
	port = ReadByte();
	Delay();
	StopI2C();
	Delay();
	Delay();

	return port;
}
void CommandDummy()
{
	StartI2C();
	Delay();
	WriteByte(0);
	Delay();
	StopI2C();
	Delay();
	Delay();
}
void CommandSetPullUps(char data)
{
	StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x02);
	Delay();
	WriteByte(0xA0);
	Delay();
	WriteByte(0x04);
	Delay();
	WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();

}
void CommandSetRelay(char address, char data)
{
	StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x02);
	Delay();
	WriteByte(address);
	Delay();
	WriteByte(0x40);
	Delay();
	WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();

}
void CommandSetCLC(char address, char data)
{
	StartI2C();
	Delay();
	WriteByte(0x90);
	Delay();
	WriteByte(0x04);
	Delay();
	WriteByte(address);
	Delay();
	WriteByte(0x30);
	Delay();
	WriteByte(data);
	Delay();
	StopI2C();
	Delay();
	Delay();

}